@@ -2,7 +2,7 @@
 # Kernel/Language/bb.pm - provides bavarian language translation
 # Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
 # --
-# $Id: bb.pm,v 1.24 2007/06/28 23:58:32 martin Exp $
+# $Id: bb.pm,v 1.25.2.2 2007/12/10 08:44:44 mh Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -13,7 +13,7 @@ package Kernel::Language::bb;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.24 $';
+$VERSION = '$Revision: 1.25.2.2 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub Data {
@@ -611,8 +611,11 @@ sub Data {
         # Template: AdminCustomerUserServiceChangeForm
         'Customer Users <-> Services Management' => 'Kundanbenutzer <-> Services Verwaltung',
         'Select the customeruser:service relations.' => 'Auswahl der Kundanbenutzer:Service Beziehungen.',
+        'Allocate services to CustomerUser' => '',
+        'Allocate CustomerUser to service' => '',
 
         # Template: AdminCustomerUserServiceForm
+        'Edit default services.' => '',
 
         # Template: AdminEmail
         'Message sent to' => 'Nachricht gesendet an',
@@ -727,7 +730,7 @@ sub Data {
         'Performance Log' => 'Performance Log',
         'This feature is enabled!' => 'Dieses Feature ist aktiv!',
         'Just use this feature if you want to log each request.' => 'Nur aktivieren wenn jede Anfrage protokolliert werden soll.',
-        'Of couse this feature will take some system performance it self!' => 'Wenn dieses Feature akitv ist, wird ist mit Leistungsdefizit zum rechnen.',
+        'Of couse this feature will take some system performance it self!' => 'Wenn dieses Feature aktiv ist, ist mit Leistungsdefizit zum rechnen.',
         'Disable it here!' => 'Hier deaktivieren!',
         'This feature is disabled!' => 'Dieses Feature ist inaktiv!',
         'Enable it here!' => 'Hier aktivieren!',
@@ -779,7 +782,7 @@ sub Data {
         '0 = no unlock' => '0 = keine Freigabe',
         'Escalation - First Response Time' => 'Eskalation - Reaktionszeit',
         '0 = no escalation' => '0 = keine Eskalation',
-        'Escalation - Update Time' => 'Eskalation - Aktuallisierungszeit',
+        'Escalation - Update Time' => 'Eskalation - Aktualisierungszeit',
         'Escalation - Solution Time' => 'Eskalation - Lsungszeit',
         'Follow up Option' => 'Nachfrage Option',
         'Ticket lock after a follow up' => 'Ticket sperren nach einem Follow-Up',
@@ -882,7 +885,7 @@ sub Data {
         'Add a new SLA.' => 'Einen neuen SLA dazua doa.',
         'SLA' => 'SLA',
         'First Response Time' => 'Reaktionszeit',
-        'Update Time' => 'Aktuallisierungszeit',
+        'Update Time' => 'Aktualisierungszeit',
         'Solution Time' => 'Lsungszeit',
 
         # Template: AdminSMIMEForm
