@@ -1,12 +1,12 @@
 # --
-# Kernel/Modules/AgentTicketEmail.pm - to compose inital email to customer
-# Copyright (C) 2001-2007 OTRS GmbH, http://otrs.org/
+# Kernel/Modules/AgentTicketEmail.pm - to compose initial email to customer
+# Copyright (C) 2001-2008 OTRS AG, http://otrs.org/
 # --
-# $Id: AgentTicketEmail.pm,v 1.24.2.1 2007/03/02 00:09:47 martin Exp $
+# $Id: AgentTicketEmail.pm,v 1.43.2.2 2008/01/25 15:55:13 ub Exp $
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/gpl.txt.
+# did not receive this file, see http://www.gnu.org/licenses/gpl-2.0.txt.
 # --
 
 package Kernel::Modules::AgentTicketEmail;
@@ -20,7 +20,7 @@ use Kernel::System::State;
 use Mail::Address;
 
 use vars qw($VERSION);
-$VERSION = '$Revision: 1.24.2.1 $';
+$VERSION = '$Revision: 1.43.2.2 $';
 $VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;
 
 sub new {
@@ -43,8 +43,6 @@ sub new {
             $Self->{LayoutObject}->FatalError(Message => "Got no $_!");
         }
     }
-    # anyway, we need to check the email syntax (removed it, because the admins should configure it)
-#    $Self->{ConfigObject}->Set(Key => 'CheckEmailAddresses', Value => 1);
     # needed objects
     $Self->{SystemAddress} = Kernel::System::SystemAddress->new(%Param);
     $Self->{CustomerUserObject} = Kernel::System::CustomerUser->new(%Param);
@@ -80,6 +78,7 @@ sub Run {
     my %GetParam = ();
     foreach (qw(AttachmentUpload
         Year Month Day Hour Minute To Cc Bcc TimeUnits PriorityID Subject Body
+        TypeID ServiceID SLAID AllUsers ResponsibleAll
         AttachmentDelete1 AttachmentDelete2 AttachmentDelete3 AttachmentDelete4
         AttachmentDelete5 AttachmentDelete6 AttachmentDelete7 AttachmentDelete8
         AttachmentDelete9 AttachmentDelete10 AttachmentDelete11 AttachmentDelete12
@@ -89,19 +88,23 @@ sub Run {
     }
     # get ticket free text params
     foreach (1..16) {
-        $GetParam{"TicketFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
-        $GetParam{"TicketFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
+        $GetParam{"TicketFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeKey$_");
+        $GetParam{"TicketFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "TicketFreeText$_");
     }
-    # get ticket free text params
-    foreach (1..2) {
-        foreach my $Type (qw(Year Month Day Hour Minute)) {
-            $GetParam{"TicketFreeTime".$_.$Type} =  $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+    # get ticket free time params
+    foreach (1..6) {
+        foreach my $Type (qw(Used Year Month Day Hour Minute)) {
+            $GetParam{"TicketFreeTime".$_.$Type} = $Self->{ParamObject}->GetParam(Param => "TicketFreeTime".$_.$Type);
+        }
+        $GetParam{'TicketFreeTime'.$_.'Optional'} = $Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_) || 0;
+        if (!$Self->{ConfigObject}->Get('TicketFreeTimeOptional'.$_)) {
+            $GetParam{'TicketFreeTime'.$_.'Used'} = 1;
         }
     }
     # get article free text params
     foreach (1..3) {
-        $GetParam{"ArticleFreeKey$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
-        $GetParam{"ArticleFreeText$_"} =  $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
+        $GetParam{"ArticleFreeKey$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeKey$_");
+        $GetParam{"ArticleFreeText$_"} = $Self->{ParamObject}->GetParam(Param => "ArticleFreeText$_");
     }
 
     if (!$Self->{Subaction} || $Self->{Subaction} eq 'Created') {
@@ -119,14 +122,12 @@ sub Run {
                     Link => '$Env{"Baselink"}Action=AgentTicketZoom&TicketID='.$Ticket{TicketID},
                 );
             }
-            # --
             # get split article if given
-            # --
             # get default selections
             my %TicketFreeDefault = ();
             foreach (1..16) {
-                $TicketFreeDefault{'TicketFreeKey'.$_} = $Self->{ConfigObject}->Get('TicketFreeKey'.$_.'::DefaultSelection');
-                $TicketFreeDefault{'TicketFreeText'.$_} = $Self->{ConfigObject}->Get('TicketFreeText'.$_.'::DefaultSelection');
+                $TicketFreeDefault{'TicketFreeKey'.$_} = $GetParam{'TicketFreeKey'.$_} || $Self->{ConfigObject}->Get('TicketFreeKey'.$_.'::DefaultSelection');
+                $TicketFreeDefault{'TicketFreeText'.$_} = $GetParam{'TicketFreeText'.$_} || $Self->{ConfigObject}->Get('TicketFreeText'.$_.'::DefaultSelection');
             }
             # get free text config options
             my %TicketFreeText = ();
@@ -159,6 +160,12 @@ sub Run {
                 %Param,
                 Ticket => \%GetParam,
             );
+            # get article free text default selections
+            my %ArticleFreeDefault = ();
+            foreach (1..3) {
+                $ArticleFreeDefault{'ArticleFreeKey'.$_} = $GetParam{'ArticleFreeKey'.$_} || $Self->{ConfigObject}->Get('ArticleFreeKey'.$_.'::DefaultSelection');
+                $ArticleFreeDefault{'ArticleFreeText'.$_} = $GetParam{'ArticleFreeText'.$_} || $Self->{ConfigObject}->Get('ArticleFreeText'.$_.'::DefaultSelection');
+            }
             # article free text
             my %ArticleFreeText = ();
             foreach (1..3) {
@@ -177,31 +184,32 @@ sub Run {
             }
             my %ArticleFreeTextHTML = $Self->{LayoutObject}->TicketArticleFreeText(
                 Config => \%ArticleFreeText,
-                Article => \%GetParam,
+                Article => {
+                    %GetParam,
+                    %ArticleFreeDefault,
+                },
             );
             # run compose modules
             if (ref($Self->{ConfigObject}->Get('Ticket::Frontend::ArticleComposeModule')) eq 'HASH') {
                 my %Jobs = %{$Self->{ConfigObject}->Get('Ticket::Frontend::ArticleComposeModule')};
                 foreach my $Job (sort keys %Jobs) {
-                        # load module
-                        if ($Self->{MainObject}->Require($Jobs{$Job}->{Module})) {
-                            my $Object = $Jobs{$Job}->{Module}->new(
-                                %{$Self},
-                                Debug => $Self->{Debug},
-                            );
-                            # get params
-                            my %GetParam;
-                            foreach ($Object->Option(%GetParam, Config => $Jobs{$Job})) {
-                                $GetParam{$_} = $Self->{ParamObject}->GetParam(Param => $_);
-                            }
-                            # run module
-                            $Object->Run(%GetParam, Config => $Jobs{$Job});
-                            # get errors
-#                            %Error = (%Error, $Object->Error(%GetParam, Config => $Jobs{$Job}));
-                        }
-                        else {
-                            return $Self->{LayoutObject}->FatalError();
+                    # load module
+                    if ($Self->{MainObject}->Require($Jobs{$Job}->{Module})) {
+                        my $Object = $Jobs{$Job}->{Module}->new(
+                            %{$Self},
+                            Debug => $Self->{Debug},
+                        );
+                        # get params
+                        my %GetParam;
+                        foreach ($Object->Option(%GetParam, Config => $Jobs{$Job})) {
+                            $GetParam{$_} = $Self->{ParamObject}->GetParam(Param => $_);
                         }
+                        # run module
+                        $Object->Run(%GetParam, Config => $Jobs{$Job});
+                    }
+                    else {
+                        return $Self->{LayoutObject}->FatalError();
+                    }
                 }
             }
             # html output
@@ -209,13 +217,18 @@ sub Run {
                 QueueID => $Self->{QueueID},
                 NextStates => $Self->_GetNextStates(QueueID => 1),
                 Priorities => $Self->_GetPriorities(QueueID => 1),
+                Types => $Self->_GetTypes(QueueID => 1),
+                Services => $Self->_GetServices(
+                    QueueID => 1,
+                ),
+                SLAs => $Self->_GetSLAs(QueueID => 1, %GetParam),
                 Users => $Self->_GetUsers(),
                 FromList => $Self->_GetTos(),
                 To => '',
                 Subject => $Self->{LayoutObject}->Output(Template => $Self->{Config}->{Subject}),
                 Body => $Self->{LayoutObject}->Output(Template => $Self->{Config}->{Body}),
                 CustomerID => '',
-                CustomerUser =>  '',
+                CustomerUser => '',
                 CustomerData => {},
                 %TicketFreeTextHTML,
                 %TicketFreeTimeHTML,
@@ -239,25 +252,12 @@ sub Run {
             );
         }
         my $NextState = $StateData{Name};
+        my $NewResponsibleID = $Self->{ParamObject}->GetParam(Param => 'NewResponsibleID') || '';
         my $NewUserID = $Self->{ParamObject}->GetParam(Param => 'NewUserID') || '';
-        my $Lock = $Self->{ParamObject}->GetParam(Param => 'Lock') || '';
-        if ($Lock) {
-            $Self->{ConfigObject}->Set(
-                Key => 'EmailDefaultNewLock',
-                Value => $Lock,
-            );
-        }
-        if ($NewUserID) {
-            $Self->{ConfigObject}->Set(
-                Key => 'EmailDefaultNewLock',
-                Value => 'lock',
-            );
-        }
         my $Dest = $Self->{ParamObject}->GetParam(Param => 'Dest') || '';
         my ($NewQueueID, $From) = split(/\|\|/, $Dest);
-        my $AllUsers = $Self->{ParamObject}->GetParam(Param => 'AllUsers') || '';
         if (!$NewQueueID) {
-            $AllUsers = 1;
+            $GetParam{AllUsers} = 1;
         }
         # get sender queue from
         my %Queue = ();
@@ -268,7 +268,7 @@ sub Run {
             $Signature = $Self->{QueueObject}->GetSignature(QueueID => $NewQueueID);
             $Signature =~ s/<OTRS_FIRST_NAME>/$Self->{UserFirstname}/g;
             $Signature =~ s/<OTRS_LAST_NAME>/$Self->{UserLastname}/g;
-            # replace user staff
+            # current user
             my %User = $Self->{UserObject}->GetUserData(
                 UserID => $Self->{UserID},
                 Cached => 1,
@@ -276,10 +276,19 @@ sub Run {
             foreach my $UserKey (keys %User) {
                 if ($User{$UserKey}) {
                     $Signature =~ s/<OTRS_Agent_$UserKey>/$User{$UserKey}/gi;
+                    $Signature =~ s/<OTRS_CURRENT_$UserKey>/$User{$UserKey}/gi;
                 }
             }
-            # cleanup all not needed <OTRS_TICKET_ tags
+            # replace other needed stuff
+            $Signature =~ s/<OTRS_FIRST_NAME>/$Self->{UserFirstname}/g;
+            $Signature =~ s/<OTRS_LAST_NAME>/$Self->{UserLastname}/g;
+            # cleanup
             $Signature =~ s/<OTRS_Agent_.+?>/-/gi;
+            $Signature =~ s/<OTRS_CURRENT_.+?>/-/gi;
+
+            # replace config options
+            $Signature =~ s{<OTRS_CONFIG_(.+?)>}{$Self->{ConfigObject}->Get($1)}egx;
+            $Signature =~ s/<OTRS_CONFIG_.+?>/-/gi;
         }
         my $CustomerUser = $Self->{ParamObject}->GetParam(Param => 'CustomerUser') ||
             $Self->{ParamObject}->GetParam(Param => 'PreSelectedCustomerUser') ||
@@ -291,7 +300,11 @@ sub Run {
         $GetParam{QueueID} = $NewQueueID;
         $GetParam{ExpandCustomerName} = $ExpandCustomerName;
         if ($Self->{ParamObject}->GetParam(Param => 'AllUsersRefresh')) {
-            $AllUsers = 1;
+            $GetParam{AllUsers} = 1;
+            $ExpandCustomerName = 3;
+        }
+        if ($Self->{ParamObject}->GetParam(Param => 'ResponsibleAllRefresh')) {
+            $GetParam{ResponsibleAll} = 1;
             $ExpandCustomerName = 3;
         }
         if ($Self->{ParamObject}->GetParam(Param => 'ClearTo')) {
@@ -324,6 +337,10 @@ sub Run {
                 QueueID => $NewQueueID || 0,
                 UserID => $Self->{UserID},
             );
+            # check required FreeTextField (if configured)
+            if ($Self->{Config}{'TicketFreeText'}{$_} == 2 && $GetParam{"TicketFreeText$_"} eq '' && $ExpandCustomerName == 0) {
+                $Error{"TicketFreeTextField$_ invalid"} = 'invalid';
+            }
         }
         my %TicketFreeTextHTML = $Self->{LayoutObject}->AgentFreeText(
             Config => \%TicketFreeText,
@@ -492,6 +509,9 @@ sub Run {
                 $Error{"Date invalid"} = 'invalid';
             }
         }
+        if ($Self->{ConfigObject}->Get('Ticket::Service') && $GetParam{SLAID} && !$GetParam{ServiceID}) {
+            $Error{"Service invalid"} = 'invalid';
+        }
         # run compose modules
         my %ArticleParam = ();
         if (ref($Self->{ConfigObject}->Get('Ticket::Frontend::ArticleComposeModule')) eq 'HASH') {
@@ -521,21 +541,33 @@ sub Run {
         }
 
         if (%Error) {
-            # --
+
+            # get services
+            my $Services = $Self->_GetServices(
+                CustomerUserID => $CustomerUser || '',
+                QueueID => $NewQueueID || 1,
+            );
+            # reset previous ServiceID to reset SLA-List if no service is selected
+            if ( !$Services->{$GetParam{ServiceID}} ) {
+                $GetParam{ServiceID} = '';
+            }
+
             # header
-            # --
             $Output .= $Self->{LayoutObject}->Header();
             $Output .= $Self->{LayoutObject}->NavigationBar();
-            # --
             # html output
-            # --
             $Output .= $Self->_MaskEmailNew(
                 QueueID => $Self->{QueueID},
-                Users => $Self->_GetUsers(QueueID => $NewQueueID, AllUsers => $AllUsers),
+                Users => $Self->_GetUsers(QueueID => $NewQueueID, AllUsers => $GetParam{AllUsers}),
                 UserSelected => $NewUserID,
+                ResponsibleUsers => $Self->_GetUsers(QueueID => $NewQueueID, AllUsers => $GetParam{ResponsibleAll}),
+                ResponsibleUsersSelected => $NewResponsibleID,
                 NextStates => $Self->_GetNextStates(QueueID => $NewQueueID || 1),
                 NextState => $NextState,
                 Priorities => $Self->_GetPriorities(QueueID => $NewQueueID || 1),
+                Types => $Self->_GetTypes(QueueID => $NewQueueID || 1),
+                Services => $Services,
+                SLAs => $Self->_GetSLAs(QueueID => $NewQueueID || 1, %GetParam),
                 CustomerID => $Self->{LayoutObject}->Ascii2Html(Text => $CustomerID),
                 CustomerUser => $CustomerUser,
                 CustomerData => \%CustomerData,
@@ -581,7 +613,17 @@ sub Run {
                     ReturnSubType => '-',
                     UserID => $Self->{UserID},
                 );
+
                 my %AclAction = $Self->{TicketObject}->TicketAclActionData();
+
+                # ticket title
+                if ( $Self->{ConfigObject}->Get('Ticket::Frontend::Title') ) {
+                    $Self->{LayoutObject}->Block(
+                        Name => 'Title',
+                        Data => { %Param, %Article },
+                    );
+                }
+
                 # run ticket menu modules
                 if (ref($Self->{ConfigObject}->Get('Ticket::Frontend::PreMenuModule')) eq 'HASH') {
                     my %Menus = %{$Self->{ConfigObject}->Get('Ticket::Frontend::PreMenuModule')};
@@ -636,7 +678,6 @@ sub Run {
                         %AclAction,
                         %Article,
                         Age => $Self->{LayoutObject}->CustomerAge(Age => $Article{Age}, Space => ' '),
-                        TicketOverTime => $Self->{LayoutObject}->CustomerAge(Age => $Article{TicketOverTime}, Space => ' '),
                     }
                 );
             }
@@ -649,10 +690,13 @@ sub Run {
             QueueID => $NewQueueID,
             Subject => $GetParam{Subject},
             Lock => 'unlock',
+            TypeID => $GetParam{TypeID} || '',
+            ServiceID => $GetParam{ServiceID} || '',
+            SLAID => $GetParam{SLAID} || '',
             StateID => $NextStateID,
             PriorityID => $GetParam{PriorityID},
             OwnerID => $Self->{UserID},
-            CustomerNo => $CustomerID,
+            CustomerID => $CustomerID,
             CustomerUser => $SelectedCustomerUser,
             UserID => $Self->{UserID},
         );
@@ -669,14 +713,30 @@ sub Run {
             }
         }
         # set ticket free time
-        foreach (1..2) {
+        foreach (1..6) {
             if (defined($GetParam{"TicketFreeTime".$_."Year"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Month"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Day"}) &&
                 defined($GetParam{"TicketFreeTime".$_."Hour"}) &&
-                defined($GetParam{"TicketFreeTime".$_."Minute"})) {
+                defined($GetParam{"TicketFreeTime".$_."Minute"})
+            ) {
+                my %Time;
+                $Time{"TicketFreeTime".$_."Year"} = 0;
+                $Time{"TicketFreeTime".$_."Month"} = 0;
+                $Time{"TicketFreeTime".$_."Day"} = 0;
+                $Time{"TicketFreeTime".$_."Hour"} = 0;
+                $Time{"TicketFreeTime".$_."Minute"} = 0;
+                $Time{"TicketFreeTime".$_."Secunde"} = 0;
+
+                if ($GetParam{"TicketFreeTime".$_."Used"}) {
+                    %Time = $Self->{LayoutObject}->TransfromDateSelection(
+                        %GetParam,
+                        Prefix => "TicketFreeTime".$_,
+                    );
+                }
                 $Self->{TicketObject}->TicketFreeTimeSet(
-                    %GetParam,
+                    %Time,
+                    Prefix => "TicketFreeTime",
                     TicketID => $TicketID,
                     Counter => $_,
                     UserID => $Self->{UserID},
@@ -761,6 +821,14 @@ sub Run {
                     UserID => $Self->{UserID},
                 );
             }
+            # set responsible (if new user id is given)
+            if ($NewResponsibleID) {
+                $Self->{TicketObject}->ResponsibleSet(
+                    TicketID => $TicketID,
+                    NewUserID => $NewResponsibleID,
+                    UserID => $Self->{UserID},
+                );
+            }
             # time accounting
             if ($GetParam{TimeUnits}) {
                 $Self->{TicketObject}->TicketAccountTime(
@@ -769,12 +837,6 @@ sub Run {
                     TimeUnit => $GetParam{TimeUnits},
                     UserID => $Self->{UserID},
                 );
-                # set lock
-                $Self->{TicketObject}->LockSet(
-                    TicketID => $TicketID,
-                    Lock => $Self->{ConfigObject}->Get('EmailDefaultNewLock'),
-                    UserID => $Self->{UserID},
-                );
             }
             # should i set an unlock?
             my %StateData = $Self->{StateObject}->StateGet(ID => $NextStateID);
@@ -886,6 +948,51 @@ sub _GetPriorities {
     return \%Priorities;
 }
 
+sub _GetTypes {
+    my $Self = shift;
+    my %Param = @_;
+    my %Type = ();
+    # get priority
+    if ($Param{QueueID} || $Param{TicketID}) {
+        %Type = $Self->{TicketObject}->TicketTypeList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%Type;
+}
+
+sub _GetServices {
+    my $Self = shift;
+    my %Param = @_;
+    my %Service = ();
+    # get priority
+    if (($Param{QueueID} || $Param{TicketID}) && $Param{CustomerUserID}) {
+        %Service = $Self->{TicketObject}->TicketServiceList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%Service;
+}
+
+sub _GetSLAs {
+    my $Self = shift;
+    my %Param = @_;
+    my %SLA = ();
+    # get priority
+    if ($Param{ServiceID}) {
+        %SLA = $Self->{TicketObject}->TicketSLAList(
+            %Param,
+            Action => $Self->{Action},
+            UserID => $Self->{UserID},
+        );
+    }
+    return \%SLA;
+}
+
 sub _GetTos {
     my $Self = shift;
     my %Param = @_;
@@ -947,6 +1054,11 @@ sub _MaskEmailNew {
     my $Self = shift;
     my %Param = @_;
     $Param{FormID} = $Self->{FormID};
+    # get list type
+    my $TreeView = 0;
+    if ($Self->{ConfigObject}->Get('Ticket::Frontend::ListType') eq 'tree') {
+        $TreeView = 1;
+    }
     # build string
     $Param{Users}->{''} = '-';
     $Param{'OptionStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
@@ -972,7 +1084,7 @@ sub _MaskEmailNew {
     my %NewTo = ();
     if ($Param{FromList}) {
         foreach (keys %{$Param{FromList}}) {
-             $NewTo{"$_||$Param{FromList}->{$_}"} = $Param{FromList}->{$_};
+            $NewTo{"$_||$Param{FromList}->{$_}"} = $Param{FromList}->{$_};
         }
     }
     if ($Self->{ConfigObject}->Get('Ticket::Frontend::NewQueueSelectionType') eq 'Queue') {
@@ -1005,9 +1117,57 @@ sub _MaskEmailNew {
             Data => \%Param,
         );
     }
-    # do html quoting
-    foreach (qw(From To Cc Bcc)) {
-        $Param{$_} = $Self->{LayoutObject}->Ascii2Html(Text => $Param{$_}) || '';
+    # prepare errors!
+    if ($Param{Errors}) {
+        foreach (keys %{$Param{Errors}}) {
+            $Param{$_} = "* ".$Self->{LayoutObject}->Ascii2Html(Text => $Param{Errors}->{$_});
+        }
+    }
+    # build type string
+    if ($Self->{ConfigObject}->Get('Ticket::Type')) {
+        $Param{'TypeStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{Types},
+            Name => 'TypeID',
+            SelectedID => $Param{TypeID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketType',
+            Data => {%Param},
+        );
+    }
+    # build service string
+    if ($Self->{ConfigObject}->Get('Ticket::Service')) {
+        $Param{'ServiceStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{Services},
+            Name => 'ServiceID',
+            SelectedID => $Param{ServiceID},
+            PossibleNone => 1,
+            TreeView => $TreeView,
+            Sort => 'TreeView',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketService',
+            Data => {%Param},
+        );
+        $Param{'SLAStrg'} = $Self->{LayoutObject}->BuildSelection(
+            Data => $Param{SLAs},
+            Name => 'SLAID',
+            SelectedID => $Param{SLAID},
+            PossibleNone => 1,
+            Sort => 'AlphanumericValue',
+            Translation => 0,
+            OnChange => "document.compose.ExpandCustomerName.value='3'; document.compose.submit(); return false;",
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'TicketSLA',
+            Data => {%Param},
+        );
     }
     # build priority string
     if (!$Param{PriorityID}) {
@@ -1025,12 +1185,6 @@ sub _MaskEmailNew {
         Format => 'DateInputFormatLong',
         DiffTime => $Self->{ConfigObject}->Get('Ticket::Frontend::PendingDiffTime') || 0,
     );
-    # prepare errors!
-    if ($Param{Errors}) {
-        foreach (keys %{$Param{Errors}}) {
-            $Param{$_} = "* ".$Self->{LayoutObject}->Ascii2Html(Text => $Param{Errors}->{$_});
-        }
-    }
     # from update
     if (!$Self->{LayoutObject}->{BrowserJavaScriptSupport}) {
         $Self->{LayoutObject}->Block(
@@ -1057,6 +1211,32 @@ sub _MaskEmailNew {
             );
         }
     }
+    # show responsible selection
+    if ($Self->{ConfigObject}->Get('Ticket::Responsible') &&
+        $Self->{ConfigObject}->Get('Ticket::Frontend::NewResponsibleSelection')) {
+        $Param{ResponsibleUsers}->{''} = '-';
+        $Param{'ResponsibleOptionStrg'} = $Self->{LayoutObject}->OptionStrgHashRef(
+            Data => $Param{ResponsibleUsers},
+            SelectedID => $Param{ResponsibleUsersSelected},
+            Name => 'NewResponsibleID',
+        );
+        $Self->{LayoutObject}->Block(
+            Name => 'ResponsibleSelection',
+            Data => \%Param,
+        );
+        if ($Self->{LayoutObject}->{BrowserJavaScriptSupport}) {
+            $Self->{LayoutObject}->Block(
+                Name => 'ResponsibleSelectionAllJS',
+                Data => { },
+            );
+        }
+        else {
+            $Self->{LayoutObject}->Block(
+                Name => 'ResponsibleSelectionAllSubmit',
+                Data => { },
+            );
+        }
+    }
     # ticket free text
     my $Count = 0;
     foreach (1..16) {
@@ -1068,6 +1248,7 @@ sub _MaskEmailNew {
                     TicketFreeKeyField => $Param{'TicketFreeKeyField'.$Count},
                     TicketFreeTextField => $Param{'TicketFreeTextField'.$Count},
                     Count => $Count,
+                    %Param,
                 },
             );
             $Self->{LayoutObject}->Block(
@@ -1080,7 +1261,7 @@ sub _MaskEmailNew {
         }
     }
     $Count = 0;
-    foreach (1..2) {
+    foreach (1..6) {
         $Count++;
         if ($Self->{Config}->{'TicketFreeTime'}->{$Count}) {
             $Self->{LayoutObject}->Block(
@@ -1154,6 +1335,18 @@ sub _MaskEmailNew {
             Data => $DataRef,
         );
     }
+    # jscript check freetextfields by submit
+    foreach my $Key (keys %{$Self->{Config}->{'TicketFreeText'}}) {
+        if ($Self->{Config}->{TicketFreeText}->{$Key} == 2) {
+            $Self->{LayoutObject}->Block(
+                Name => 'TicketFreeTextCheckJs',
+                Data => {
+                    TicketFreeTextField => "TicketFreeText$Key",
+                    TicketFreeKeyField => "TicketFreeKey$Key",
+                },
+            );
+        }
+    }
     # get output back
     return $Self->{LayoutObject}->Output(TemplateFile => 'AgentTicketEmail', Data => \%Param);
 }
